
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture; //input image

uniform highp vec2 uTexelSize;
uniform highp float uScanRadius; //number of texels to check around texel of interest

highp float rand(highp vec2 co)
{
    highp float a = 12.9898;
    highp float b = 78.233;
    highp float c = 43758.5453;
    highp float dt= dot(co.xy ,vec2(a,b));
    highp float sn= mod(dt,3.14);
    return abs(fract(sin(sn) * c));
}

void main()
{
    //input image
    highp vec4 textureColor0 = texture2D(inputImageTexture, textureCoordinate);
   
    //random sample search diameter
    highp float scanDiam = 2.0*uScanRadius;
    //compute the offset corner of the scan area
    highp vec2 scanCorner = vec2(uScanRadius*uTexelSize.x, uScanRadius*uTexelSize.y);
 
    highp vec4 rs = vec4(textureCoordinate.y,textureColor0.r,textureCoordinate.x,textureColor0.b);

    highp vec2 randSeed1 = rs.rg;
    highp vec2 randSeed2 = rs.rb;
    highp vec2 randSeed3 = rs.rr;
    highp vec2 randSeed4 = rs.bb;
    highp vec2 randSeed5 = rs.bg;
    highp vec2 randSeed6 = rs.br;
    highp vec2 randSeed7 = rs.gr;
    highp vec2 randSeed8 = rs.gb;
    highp vec2 randSeed9 = rs.gg;
    highp vec2 randSeed10 = rs.ra;
    
    highp vec2 rnum1 = vec2(rand(randSeed1), rand(randSeed2));
    highp vec2 rvec1 = scanDiam*uTexelSize*rnum1;
    highp vec2 rCoor1 = textureCoordinate - scanCorner + rvec1;
    highp vec4 textureColor1 = texture2D(inputImageTexture, rCoor1);
    highp float brightness1 = dot(textureColor1.rgb, vec3(1.0, 1.0, 1.0))/3.0;
    
    highp vec2 rnum2 = vec2(rand(randSeed3), rand(randSeed4));
    highp vec2 rvec2 = scanDiam*uTexelSize*rnum2;
    highp vec2 rCoor2 = textureCoordinate - scanCorner + rvec2;
    highp vec4 textureColor2 = texture2D(inputImageTexture, rCoor2);
    highp float brightness2 = dot(textureColor2.rgb, vec3(1.0, 1.0, 1.0))/3.0;
    
    highp vec2 rnum3 = vec2(rand(randSeed7), rand(randSeed8));
    highp vec2 rvec3 = scanDiam*uTexelSize*rnum3;
    highp vec2 rCoor3 = textureCoordinate - scanCorner + rvec3;
    highp vec4 textureColor3 = texture2D(inputImageTexture, rCoor3);
    highp float brightness3 = dot(textureColor3.rgb, vec3(1.0, 1.0, 1.0))/3.0;
    
    highp vec2 rnum4 = vec2(rand(randSeed5), rand(randSeed6));
    highp vec2 rvec4 = scanDiam*uTexelSize*rnum4;
    highp vec2 rCoor4 = textureCoordinate - scanCorner + rvec4;
    highp vec4 textureColor4 = texture2D(inputImageTexture, rCoor4);
    highp float brightness4 = dot(textureColor4.rgb, vec3(1.0, 1.0, 1.0))/3.0;
    
    highp vec2 rnum5 = vec2(rand(randSeed9), rand(randSeed10));
    highp vec2 rvec5 = scanDiam*uTexelSize*rnum5;
    highp vec2 rCoor5 = textureCoordinate - scanCorner + rvec5;
    highp vec4 textureColor5 = texture2D(inputImageTexture, rCoor5);
    highp float brightness5 = dot(textureColor5.rgb, vec3(1.0, 1.0, 1.0))/3.0;
    
    highp vec4 newColor = (textureColor1 + textureColor2 + textureColor3 + textureColor4 + textureColor5)/5.0;
    highp float newBright = max(max(max(max(brightness1,brightness2),brightness3),brightness4),brightness5);
    newBright = (brightness1 + brightness2 + brightness3 + brightness4 + brightness5)/5.0;
    highp float brightness0 = dot(textureColor0.rgb, vec3(1.0, 1.0, 1.0))/3.0; //brightness of the original image
    highp vec4 outColor;

    highp float sig = sign(newBright - brightness0);
    sig = sig + (1.0 - abs(sig));
    sig = sig / 2.0 + 0.5;
    outColor = (0.4*newColor*255.0 + 0.6*textureColor0*255.0 + (sig) * 0.65*pow(abs(brightness0*255.0 - newBright*255.0), 0.8)) / 255.0;
    
    gl_FragColor = outColor;
}
